/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//    Dump frames from shared memory partition.
//
#include "lsmp_con.hh"
#include "Interval.hh"
#include "Time.hh"
#include "SigFlag.hh"
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fstream>
#include <iostream>
#include <string>
#include <cmath>
#include "FrameL.h"
#include "CmMessage.h"

using namespace std;

extern "C" 
{
void FdSendFrame(const char* cmName,
		 const char* destination,
		 struct FrameH *frame,
		 int debugPeriod);
}

//======================================  Parameter constants
const int def_stat_count = 100;


//======================================  Main function.
int 
main(int argc, const char *argv[]) {
    string partition = "LHO_hoft_H1";
    string destination = "H1In";
    int debugPeriod = 1000; //--in frame units

    for (int i=1; i<argc; ++i) {
        string argi = argv[i];
        if (argi == "-d" || argi == "--destination") {
            destination = argv[++i];
        } else if (argi == "-p" || argi == "--partition") {
            partition = argv[++i];
        } else if (argi == "-i" || argi == "--infoPeriod") {
            debugPeriod = atoi(argv[++i]);
         } else {
            cerr << "Invalid argument: " << argi << endl;
            exit(2);}}
 
    printf("Partition is %s: destination is: %s\n",
	   partition.c_str(), destination.c_str());

    //---- To_destination may be a better process name 
    //----  but Cm does not support "_"
    string cmName = "To";
    cmName += destination;
    if ( !CmMessageOpenServer (const_cast<char*>(cmName.c_str()))) {
	printf(" Cannot open Cm\n");
	exit(1);
    }
    printf(" Cm OK\n");
    fflush(stdout);

    CmConnectSetSocketSize(262144, 262144, "*");
    
    long int lastFrame = time(NULL);
    FRBOOL running = FR_YES;

    //----------------------------------  Open the share memory partition
    LSMP_CON cons(partition.c_str(), 0);

    //----------------------------------  Handle signals
    SigFlag mTerm;
    mTerm.add(SIGTERM);
    mTerm.add(SIGHUP);
    mTerm.add(SIGINT);

    //----------------------------------  Open the output file
    while (!mTerm) {
        CmMessageWaitWithTimeout  (.1);

	//------------------------------  Get a buffer
	const char* b = cons.get_buffer(NOWAIT);
	if (!b) {
            sleep(1);
	    if((time(NULL)-lastFrame > 60) && (running == FR_YES)) {
               running = FR_NO;
               printf("No frames received since %ld\n",time(NULL)-lastFrame);}
	    continue;
	}
        if(running == FR_NO) {
            printf("Frame are back after %ld s.\n",time(NULL)-lastFrame);
            running = FR_YES;}
        lastFrame = time(NULL);

	//----------------------------------  get the length and gps time
	long length = cons.getLength();

	FrameH* frame = FrameReadFromBuf((char*) b, length, -1);
	
        FdSendFrame (cmName.c_str(), 
		     destination.c_str(), 
		     frame, 
		     debugPeriod);

	//----------------------------------  Write a frame.
	cons.free_buffer();
    }
    return 0;
}
